module type BUNDLE_TYPE =
sig
  include Xset.XSET_TYPE
  val are_compatible : t -> t -> bool
  val conform : t -> t -> t
  val modulo_features : t -> t -> t
end

module Make
  (D : Delim.DELIM_TYPE)
  (F : Feature.FEATURE_TYPE) =
struct

  module Fb = Xset.Make(D)(F)
  include Fb

  let pair fb1 fb2 = union fb1 fb2

  let are_compatible fb1 fb2 = 
    let okForThisFeature f b = 
      if b then 
	(is_empty
	  (filter 
 	    (fun x -> not (F.are_compatible f x))
	    fb2))
      else raise Exit
    in
    try (fold okForThisFeature fb1 true) 
    with Exit -> false
      
  let get_features_with_id id fb = 
    filter (fun f -> F.same_id id f) fb

  (* this function removes any features in fb1 that do not 
     have a matching id in fb2. The value of the features are ignored. *)
  let modulo_features fb1 fb2 =
    fold (fun f2 new_fb -> 
      union (get_features_with_id f2 fb1) new_fb)
      fb2
      empty

  (*
    In conform, fb1 is thought of as the input and fb2 the
    (decided) output.
    This function returns a feature bundle which is a union of fb1 and
    fb2 with feature values of fb2 overriding those of fb1.
    E.g. conform [+a,+b,+c] [-a,+b] = [-a,+b,+c] 
  *)
      
  let conform fb1 fb2 = 
    let fb1_to_keep = fold 
      (fun f1 fb -> 
	let fset = get_features_with_id f1 fb2 in
	if is_empty fset 
	then add f1 fb
	else if F.compare f1 (choose fset) = 0 
	then add f1 fb
	else fb
      ) fb1 empty
    in 
    union fb1_to_keep fb2

end
