(** Bundle implements feature bundles as sets of features where a feature is
    the type of element defined in {!Feature.Make}.

    author : Jeff Heinz

    last updated : Spetember 2006

*)


(** The output signature of {!Bundle.Make}. *)

module type BUNDLE_TYPE = 
sig

  (** {2 Standard set functions} *)

  (** These are the functions found in [Xset.XSET_TYPE] (Tools Documentation). *)
  include Xset.XSET_TYPE      
    
    
  (** {2 Additional functions} *)

  (** Two feature bundles are compatible iff all of their features are compatible. *)
  val are_compatible : t -> t -> bool
    
  (** [conform fb1 fb2] returns a feature bundle which is the union of [fb1] and
      [fb2] except that the values of the features in [fb2] overrride those in
      [fb1]. *)
  val conform : t -> t -> t
    
  (** [modulo_features fb1 fb2] returns a feature bundle which removes any
      features in fb1 that do not have a matching id in fb2. The value of the
      features are ignored. *)
  val modulo_features : t -> t -> t
    
end


(** Bundle.Make takes a module of [Delim.DELIM_TYPE] (see Tools documentation)
    and a module of {!Feature.FEATURE_TYPE} and  returns a module of
    {!Bundle.BUNDLE_TYPE}. *)

module Make
  (Delim : Delim.DELIM_TYPE)
  (Feature: Feature.FEATURE_TYPE)
  :
  BUNDLE_TYPE with type elt = Feature.t
