

(** 
    This is documentation for the command line executable "fdo".

    author: Jeff Heinz
 
    last updated: July 30, 2006


 This page explains how to use the [fdo] command line executable.  
 Feature implements a feature system over a set of elements. Sequences of 
 elements (e.g. words) can be converted to sequences of feature bundles 
 and vice versa.  Additional programs allow one to compute the natural 
 classes, find the partitions by natural class, compute a measure of 
 similarity based on natural class (Frisch, Pierrehumbert, and Broe 2004), as well 
 as find the smallest natural class containing two elements (i.e. minimal 
 generalization (Albright and Hayes (2002, 2003)). 

References:
    - Frish S., J. Pierrehumbert, and M. Broe. 2004. Similarity Avoidance and
    the OCP. Natural Language and Linguistic Theory 22:179-228.
    - Albright, Adam and Bruce Hayes. 2002. Modeling English past tense
    intuitions with minimal generalization. SIGPHON 6: Proceedings of the
    Sixth Meeting of the ACL Special Interest Group in Computational
    Phonology. pages 58-69.
    - Albright, Adam and Bruce Hayes. 2003. Rules vs. Analogy in English Past
    Tenses:  A Computational/Experimental Study. Cognititon 90:119-161.

 {3 Usage} 

 {2 Invocation} 

 Like other OCaML executables, the program is run by typing [./fdo] or
    [ocamlrun fsa] at the command prompt.  If you compiled using [ocamlopt]
    then you can just type [fdo]. Generally it is invoked as follows.
    
    [fdo subcommand] {i options}
    
 {2 Subcommands} 

   The output of the [fdo] command is always written to standard output.
   Therefore it is possible to pipe it [|] to other commands or direct it [>]
   to a file.  

   By itself [fdo] will not do anything-- the argument [subcommand] is mandatory.
   The [subcommand] arguments recognized by [fdo] are shown in the list below.
   The subcommand is illustrated without any flag arguments assuming that
   [file] contains an FSA. A brief description is also given.
    - [w2fb] : [fdo w2fb] {i options} converts the input from words to
    sequences of feature bundles. The option [-f featurefile] is required. If
    no input file is specified, [fdo] reads from standard input.
    - [fb2w] : [fdo w2fb] {i options} converts the input from a sequence of
    feature bundles to words. The option [-f featurefile] is required. If
    no input file is specified, [fdo] reads from standard input. 
    - [fbmod] : [fdo fbmod] alters every feature bundle in the input uniformly
    by keeping only those features that are specified in [-fn
    feature_name_list] where [feature_name_list] is a lsit of features (see
    [-fn] below).
    - [nc] : [fdo nc] {i options} constructs the natural classes as a map from
    sets of segments (natural classes) to a list of feature bundles. The
    feature bundles in the list pick out the natural class. The option [-f
   featurefile] is required.
    - [sim] : [fdo sim] {i options} computes the natural class similarity
    measure for every pair of segments in the feature system. The options [-f
    featurefile] and [-ncm ncmap] are required.
    - [mg] : [fdo mg] {i options} computes the smallest natural class
    containing two elements for every pair of elements in the feature system.
    The options [-f featurefile] and [-ncm ncmap] are required.
    

 {2 Options} 


 Following the first argument (the main command) you can optionally list as many of the following
    flags as you like (separated by spaces):
    - "-i" : [-i inputfile] specifies the file which  
    - "-f" : [-f featurefile] specifies where the file where the feature system, written as a table, is.
    - "-xd" : [-xd delim] specifies how segments are delimited in the string representation of a word (default [""]).
    - "-xs" : [-xs delim] specifies how segments are delmited in the string representation of a set  (default [","])..
    - "-fbd" : [-fbd delim] specifies how feature bundles are delimited in the string representation of a sequence of feature bundles  (default ["."])..
    - "-fn" : [-fn namelist] specifies a list of feature names to keep in [fbmod].  Names are delimited with commas ([","]).
    - "-ncm" : [-ncm ncmap_file] specifies where the file containing the natural class map is.
    - "-s" : [-s] requires the natural class map to be constructed with the most specific feature bundle associated with each natural class. 
    - "-t" : [-t] requires the natural class map to be constructed with every natural class associated with every feature bundle which picks it out.
    - "-lim":[n] specifies the least number of partitions by natural class the computation must obtain before halting.

{3 Hints}

Here are some hints:
    - [fb2w] Use standard the unix commands [sort] and  [uniq] to remove
    duplicates which are formed when different sequences of feature bundles 
    generate the same word.


*)


