module type FEATURE_TYPE =
sig
  include X.X_TYPE
  type id
  type value
  val value : t -> value
  val id : t -> id
  val make : value -> id -> t
  val same_id : t -> t -> bool
  val are_compatible : t -> t -> bool    
end

module Make
  (Name: X.X_TYPE)
  (Value: Value.VALUE_TYPE) =
  
struct
  let name = "Feature"
    
  type id = Name.t
  type value = Value.t
      
  type t = {id:id;value:value} 
      
  let make v id = {id=id;value=v}
  let value f = f.value
  let id f = f.id
    
  let same_id f1 f2 = f1.id = f2.id
    
  let compare = compare
    
  let of_string f = 
    (* looks like +hungry. *)
  let v,n = Mstring.split_at 0 f in
  make (Value.of_string v) (Name.of_string n)
    
  let to_string f = (Value.to_string f.value)^(Name.to_string f.id)
    
  let print ?oc:(oc=stdout) f = 
    output_string oc (to_string f);
    output_string oc "\n";
    flush oc
      
  let print_ ?oc:(oc=stdout) f = 
    output_string oc (to_string f);
  flush oc
    
  let are_compatible x y =
    if not (Name.compare x.id y.id = 0) then true
    else if (Value.compare x.value y.value = 0) 
    then true 
    else false

  let pair f1 f2 =
    make Value.max (Name.of_string ("_"^(to_string f1)^"_"^(to_string f2)))
      
end
