(**

Feature implements features as value-name pairs. 

author:Jeff Heinz

last updated : October 1, 2006

*)


(** The input signature to {!Feature.Make}. *)


module type FEATURE_TYPE =
sig
  include X.X_TYPE
    (** The following list summarizes the function in [X.X_TYPE].
	- Type [t] is a record consisting of an [id] and [value].
	- [name] is ["Feature"]. 
	- [compare] is the same as [Pervasives.compare]. 
	- [pair f1 f2] returns the max value for a new feature id made from
	concatenating the strings representations of [f1] and [f2]. For
	example, [pair f1 f2] where (the string representation of) f1 is "+hungry" and f2 is "-thirsty"
	returns a feature which is "+_+hungry_-thirsty".
	- [of_string] assumes that the value is the first character of the
	string. Obviously this should be rewritten if floats or integers
	larger than 9 are used as values.
	- [to_string] writes the feature as a string.
	- [print] prints a feature followed by an endline.
	- [print_] prints a feature but no endline. *)

  (** The id of the feature, such as a name. *)
  type id

  (** The value of the feature, such as a name. *)
  type value
 
  (** Returns the value of a feature. *)
  val value : t -> value

  (** Returns the name of a feature. *)
  val id : t -> id

  (** Returns a feature from a value and a name. *)
  val make : value -> id -> t

  (** [same_id f1 f2] returns [true] iff [f1] and [f2] have the same [id]. *)
  val same_id : t -> t -> bool

  (** Two features are compatible if they have different ids or if they have
      the same id and the same value.  In other words, features are not
      compatible only if they have the same ids but different values. *)
  val are_compatible : t -> t -> bool 

end


  
(** The functor takes the two modules below and returns a module of FEATURE_TYPE. *)

module Make 
  (Name: X.X_TYPE)
  (Value: Value.VALUE_TYPE) 
  : FEATURE_TYPE with type id = Name.t
		 and type value = Value.t
